﻿/* A custom treeview implement the interface "nsITreeView" 
   Created by Zoominla, 02-24-2009         
*/
if(typeof(SIMPLE_TREEVIEW_INCLUDE) != "boolean") {
	SIMPLE_TREEVIEW_INCLUDE = true;
    
const C_i = Components.interfaces;
function voidFunc() {
  // do nothing
}
function returnFalse() {
  return false;
}

/**
 * Create a TreeRow associated with a given object.
 *
 * @param aObject    The raw object to bind to.
 * @param aTreeView  The nsITreeView that owns this row.
 */
function TreeRow(aObject, aTreeView) {
  this._object = aObject;
  this._treeView = aTreeView;
}

TreeRow.prototype = {
  /**
   * Does this row have any child rows?
   */
  get isContainer() {
    return false;
  },

  /**
   * Presumed false, since this shouldn't get called except if this.isContainer.
   */
  get isContainerEmpty() {
    return false;
  },

  toString: function toString() {
    return "[object TreeRow (" + this._object + ")]";
  }
};

/**
 * Define a tree view for a XUL tree, based on a hierarchy of objects.
 *
 * @param aChildGetter A JS function that returns a list of child objects for any given object.
 */
function ClassTreeView() {
  this._rows = [];
  this._hasSetTree = false;

  // nsITreeView
  this.selection = null;
}

ClassTreeView.prototype = {
  /**
   * Add a top-level object to the tree.
   *
   * @param aObject The object to add.
   * @param aOpen   Boolean, true if the object's row should be open.
   */
  addTopObject: function addTopObject(aObject) {
    var row = new TreeRow(aObject, this);
    this._rows.push(row);
  },

  /**
   * Get a TreeRow based on its index.
   *
   * @param aIndex The index of the row to get.
   *
   * @returns The TreeRow.
   */
  getRow: function getRow(aIndex) {
    return this._rows[aIndex];
  },

  /**
   * Get a TreeRow based on the object associated with it.
   *
   * @param aObject The object to find a row for.
   *
   * @returns The TreeRow.
   */
  getRowForObject: function getRowForObject(aObject) {
    for each (var row in this._rows)
    {
      if (row._object == aObject)
        return row;
    }
    return null;
  },

  /**
   * Get the object associated with a TreeRow.
   *
   * @param The TreeRow to get the object from.
   *
   * @returns The object.
   */
  getObjectForRow: function getObjectForRow(aRow) {
    return aRow._object;
  },

  getObjectForIndex: function getObjectForIndex(aIndex) {
    return this.getObjectForRow(this.getRow(aIndex));
  },

  // nsITreeView
  get rowCount() {
    return this._rows.length;
  },

  getRowProperties: voidFunc,
  getCellProperties: voidFunc,
  getColumnProperties: voidFunc,

  isContainer: returnFalse,
  isSeparator: returnFalse,
  isSorted: returnFalse,
  canDrop: returnFalse,
  drop: voidFunc,
  getImageSrc: voidFunc,
  getProgressMode: voidFunc,
  getCellValue: voidFunc,

  getCellText: function getCellText(aIndex, aColumn) {
    if(aColumn.id == "RowNo") {
      return aIndex + 1;
    }
    var obj = this.getObjectForIndex(aIndex);
    return obj[aColumn.id];
  },
    
  setTree: function(treeBox) {
    if (!treeBox)
      return;
    if (!this._hasSetTree) {
      this._hasSetTree = true;
    }
    this.treeBox = treeBox;
  },

  cycleHeader: voidFunc,
  selectionChanged: voidFunc,
  cycleCell: voidFunc,
  isEditable: returnFalse,
  isSelectable: returnFalse,
  setCellValue: voidFunc,
  setCellText: voidFunc,

  // These are dead functions.
  performAction: voidFunc,
  performActionOnRow: voidFunc,
  performActionOnCell: voidFunc,

  // Diagnostic.
  toString: function toString() {
    var msg = "";
    var indentStr = "";
    function indent() {
      indentStr += "  ";
    }
    function outdent() {
      indentStr = indentStr.substr(2);
    }
    function addLine(aLine) {
      msg += (indentStr + aLine + "\n");
    }

    addLine("{"); indent(); // begin toString()
    addLine("_rows: ["); indent(); // begin _rows

    var count = this.rowCount;
    var colCount = this._cellGetters.length;
    for (var i = 0; i < count; i++) {
      addLine("{"); indent(); // begin individual row
      addLine("rowIndex: " + i + ",");
      addLine("parentIndex: " + this.getParentIndex(i) + ",");
      addLine("level: " + this.getLevel(i) + ",");
      addLine("isContainer: " + this.isContainer(i) + ",");
      addLine("isContainerOpen: " + this.isContainerOpen(i) + ",");
      addLine("isContainerEmpty: " + this.isContainerEmpty(i) + ",");

      addLine("_cells: ["); indent(); // begin cells;
      for (var j = 0; j < colCount; j++) {
        var cellValue = "'" + this.getCellText(i, {index: j}) + "'";
        if (j < colCount - 1)
          cellValue += ",";
        addLine(cellValue);
      }
      outdent(); addLine("],"); // end _cells

      addLine("hasNextSibling: " + this.hasNextSibling(i, 0));

      outdent();
      var endChar = "}";
      if (i < count - 1)
        endChar += ","
      addLine(endChar); // end individual row
    }
    
    outdent(); addLine("]"); // end _rows
    outdent(); addLine("}"); // end toString();

    return msg;
  },

  // nsISupports
  QueryInterface: function QueryInterface(aIID) {
    if (aIID.equals(C_i.nsITreeView) ||
        aIID.equals(C_i.nsISupports))
      return this;
    throw Components.results.NS_ERROR_NO_INTERFACE;
  }
};

const EXPORTED_SYMBOLS = ["ClassTreeView"];

}